/* MOUSE ANALYSIS
;;;; BASIC PROTOCOL CALCULATER
;;;; CALCULATES X AND Y MOVEMENT BY VALUES GIVEN BY MOUSE ATTATCHED RADIALLY
;;;; (c) David Reis, Wed Mar 16, 2005 */

#include <math.h>
#include <avr/io.h>

#include "includes/i2c.h"
#include "includes/ps2.h"
#include "includes/general.h"

#define CIRCLEVALUE 314 // rad of circle mouse moves in
#define FIELDWIDTH 1220 // width of the room
#define FIELDLENGTH 1830 // length of the room

#define ZWEI_PI_DURCH_360 0.01745329252 // rad -> deg
#define GRADUMRECHNUNG 1.146498			// 360 / CIRCLEVALUE
#define COMPUMRECHNUNG 0.8152866242	    // 512 / CIRCLEVALUE
#define XUMRECHNUNG 0.2090163934		// 255 / FIELDWIDTH
#define YUMRECHNUNG 0.1393442623		// 255 / FIELDLENGTH

int main(void)
{
	// read from PB, write to PC and PD	
	DDRB = 0x00;
	DDRC = 0xFF;
	DDRD = 0xFF;
	
	// all lines high before comm
	PORTC = 0XFF;
	PORTD = 0xFF;
	
	ps2_send(243);	// set samle rate
	ps2_send(10);	// 10 samples / sec
	ps2_send(232);	// set resolution
	ps2_send(0);	// 1 tick / mm
			
	while(1)
	{
		signs = ps2_get(); // first packet is signs
		xMouse = ps2_get(); // second x movement -> turning
		yMouse = ps2_get(); // third y movement -> driving

		if ( signs & 64 ) { yMouse += 255; } // if y axis overflow, add 255 mm
		if ( signs & 128 ) { xMouse += 255; } // if x axis overflow, add 255 mm

		if ( signs & 32 ) { yMouse *= -1; } // extract y sign
		if ( signs & 16 ) { xMouse *= -1; } // extract x sign

		turnTotal -= xMouse; // x axis actually outputs values contrary to turn!
		
		// always get turnTotal to 0..CIRCLEVALUE
		if(turnTotal < 0)
			{ turnTotal += CIRCLEVALUE; }
		else
			{ turnTotal %= CIRCLEVALUE; }

		// by trigonometry (sine), get x part of movement and add it to xDistTotal
		xDistTotal += roundSignedInteger( yMouse * sin( (turnTotal * GRADUMRECHNUNG) * ZWEI_PI_DURCH_360) );
		// by trigonometry (cosine), get y part of movement and add it to yDistTotal
		yDistTotal += roundSignedInteger( yMouse * cos( (turnTotal * GRADUMRECHNUNG) * ZWEI_PI_DURCH_360) );

		// get X and Y down to 0..255
		xOut = roundSigned16BitInteger( xDistTotal * XUMRECHNUNG );
		yOut = roundSigned16BitInteger( yDistTotal * YUMRECHNUNG );
		
		// get turn to -255..255
		turnOut = turnTotal;
		if ( turnOut > 157 )
		{
			turnOut -= 157;
			turnOut *= -1;
		}
		turnOut = roundSigned16BitInteger( turnOut * COMPUMRECHNUNG );
	
		// send abs of turn and use PD,2 as sign bit
		if ( turnOut < 0 )
		{
			sbi ( PORTD, 2 );
			turnOut *= -1;
		} else { 
			cbi ( PORTD, 2 );
		}

		i2c_send(34, turnOut); // put all values to I2C, 32..34 are 0100 + 000..010
		i2c_send(32, xOut);
 		i2c_send(33, yOut);
	}

	return 0;
}
