#include <avr/io.h>
#include "general.h"
#include "i2c.h"

// I2C stop condition
void i2c_stop(void)
{
	cbi(PORTC, i2c_DATA);
	sbi(PORTC, i2c_CLOCK);
	waitSomeOps();	
	sbi(PORTC, i2c_DATA);
}

// I2C start condition
void i2c_start(void)
{
	sbi(PORTC, i2c_DATA);
	sbi(PORTC, i2c_CLOCK);
	cbi(PORTC, i2c_DATA);
	waitSomeOps();
	cbi(PORTC, i2c_CLOCK);
}

// send 8 bits to I2C
void i2c_putbyte(uint8_t data)
{
	for(statusBits.cycleBit = 0; statusBits.cycleBit < 8; statusBits.cycleBit++)
	{
		if( data & 128 ) // MSB goes first
			{ sbi(PORTC, i2c_DATA); }
		else
			{ cbi(PORTC, i2c_DATA); }
		data <<= 1; // shift left -> get next bit to MSB
		waitSomeOps();
		sbi(PORTC, i2c_CLOCK); // send clock pulse
		waitSomeOps();
		cbi(PORTC, i2c_CLOCK);
		waitSomeOps();
	}
	sbi(PORTC, i2c_DATA);
	cbi(DDRC, i2c_DATA); // read data line
	loop_until_bit_is_clear(PINC, i2c_DATA); // wait for ACK
	waitSomeOps();
	sbi(PORTC, i2c_CLOCK); // RACK wick clock pulse
	waitSomeOps();
	sbi(DDRC, i2c_DATA); // write to data
	cbi(PORTC, i2c_CLOCK);
	waitSomeOps();
}

// send 8 bits to address and append R/W bit (WRITE) to address
void i2c_send(uint8_t address, uint8_t data)
{
	i2c_start();
	address <<= 1; // write to bus = 0 as LSB
	i2c_putbyte(address);
	i2c_putbyte(data);
	i2c_stop();
}
