#include "ps2.h"
#include <avr/io.h>
#include "general.h"

// wait until clock goes low
void waitClockLow()
{
	loop_until_bit_is_clear(PINB, ps2_CLOCK_get);
}

// wait for clock to go high
void waitClockHigh()
{
	loop_until_bit_is_set(PINB, ps2_CLOCK_get);
}

// discard a bit sent by device
void discardBit()
{
	waitClockLow();
	waitClockHigh();
}

// send 8 bit to device (commands)
void send8bit(uint8_t data)
{
	for(statusBits.loopCounter = 0; statusBits.loopCounter < 8; statusBits.loopCounter++)
	{
		waitClockLow(); // wait until device is ready to read
		if ( data & 1 ) // LSB goes first
			{ sbi ( PORTD, ps2_DATA_put ); }
		else
			{ cbi ( PORTD, ps2_DATA_put ); }
		waitClockHigh(); // let remain stable until next ready
		
		data >>=1; // shift next bit into LSB
	}
}

// discard 11 bits sent by device (ACK signal)
void wait11bits()
{
	for(statusBits.loopCounter = 0; statusBits.loopCounter < 11; statusBits.loopCounter++)
	{
		discardBit();
	}
}

/* get parity from data to send and put it to bus */
void sendParity( uint8_t data )
{
	waitClockLow(); // wait until ready

	if ( data & 1 ) // read LSB and set parity to bus
		{ cbi ( PORTD, ps2_DATA_put ); }
	else
		{ sbi ( PORTD, ps2_DATA_put ); }
	
	waitClockHigh();
}

void openHostToDeviceComm()
{
	cbi(PORTD, ps2_CLOCK_put); // pull clock line low and wait							
	waitSomeOps();
	cbi(PORTD, ps2_DATA_put); // pull data line low = ready signal						
	waitSomeOps();															
	sbi(PORTD, ps2_CLOCK_put); // get clock line high = open comm						
	waitClockLow(); // wait for ready signal, leave data high as start bit				
}

void ps2_send(uint8_t byteData)
{
	openHostToDeviceComm(); // open communication
	send8bit(byteData); // put command to bus
	sendParity(byteData); // put parity					
	waitClockLow();	// wait until device ready			
	sbi(PORTD, ps2_DATA_put); // send stop bit							
	waitClockHigh(); // remain stable
	waitClockLow();
	waitClockHigh(); // recieve packet ACK				
	wait11bits(); // receive command ACK		
}

// get 8 bits from PS/2
uint8_t ps2_get()
{
	waitClockLow(); // wait for handshake
	discardBit();   // start bit

	uint8_t receivedData = 0;
		
	// get 8 data bits into received

	for (statusBits.loopCounter = 0; statusBits.loopCounter < 8; statusBits.loopCounter++)
	{
		waitClockLow();
			
		// get data bit of PORTB into MSB of received
	
		receivedData >>= 1;
		if ( bit_is_set(PINB, 1) ) { receivedData += 128; }
			
		waitClockHigh();
	}
			
	discardBit(); // parity
	discardBit(); // stop bit
	
	return receivedData;
}
